var EDITOR = (function () {

    var FPS = 10,
        BPS = 30, // brushes per second
        CELL_SIZE = 10, 
        canvas,
        ctx,    
        renderTimer,
        brushTimer,
        area = {},
        cells = [],
        protos = [],
        idCount = 1,
        brushSize = 1,
        pieruMode = false,
        currentProto = 0,
        currentType = 0,
        mouseDown = false,
        mouseCellX,
        mouseCellY,
        linesRendered = false;

    var render = function () {
	renderRect(0, 0, area.height, area.width);
    };

    var renderCell = function (cell, j, i) {
        var monster;
	var item;

        ctx.fillStyle = cell.ground.color;
        ctx.fillRect(j*CELL_SIZE + 1, i*CELL_SIZE + 1, CELL_SIZE - 1, CELL_SIZE - 1);
        monster = cell.monster;
        if (monster !== null) {
            ctx.fillStyle = monster.color;
            ctx.fillRect(j*CELL_SIZE + 1, (i+1)*CELL_SIZE-3, 3, 3);
        }
        item = cell.item;
        if (item !== null) {
            ctx.fillStyle = item.color;
            ctx.fillRect((j+1)*CELL_SIZE-3, (i+1)*CELL_SIZE-3, 3, 3);
	}	
    };

    var renderRect = function (startX, startY, width, height) {
        for (var i = startX; i < startX + width; i++) {
            for (var j = startY; j < startY + height; j++) {
                var cell = cells[area.width*i + j];
		renderCell(cell, j, i);
            }
        }

	if (!linesRendered) {
	    linesRendered = true;

            ctx.fillStyle = "#000000";
            for (var i = startX; i < startX + width + 1; i++) {
                ctx.fillRect(i*CELL_SIZE, 0, 1, canvas.height);
            }
            for (var i = startY; i < startY + height + 1; i++) {
                ctx.fillRect(0, i*CELL_SIZE, canvas.width, 1);
            }
	}
    };

    var initCells = function () {
        for (var i = 0; i < area.height; i++) {
            for (var j = 0; j < area.width; j++) {
                cells[area.width*i + j] = {ground: nameToProto("grass"),
                                           monster: null,
                                           item: null};
            }
        }  
    };

    var nameToProto = function (name) {
        for (var i = 0; i < protos.length; i++) {
            if (protos[i].name === name) {
                return protos[i];
            }
        }
        return null;
    };

    var charToProto = function (keyCharacter) {
        for (var i = 0; i < protos.length; i++) {
            if (protos[i].keyCharacter === keyCharacter) {
                return protos[i];
            }
        }
        return null;
    }

    var genColor = function (id) {
        var red = Math.max(16, (id*40 % 255)).toString(16);
        var green = Math.max(16, Math.abs(255 - id*40) % 255).toString(16);
        var blue = Math.max(16, id*15 % 255).toString(16);

        return "#" + red + green + blue;
    };

    var initArea = function (width, height) {
        area.width = parseInt(width);
        area.height = parseInt(height);

        canvas = $('#area')[0];
        canvas.width = width*CELL_SIZE + 1;
        canvas.height = height*CELL_SIZE + 1;
    
        ctx = canvas.getContext("2d");

        if (renderTimer === undefined) {
            renderTimer = setInterval(render, 1000);
        }
        if (brushTimer === undefined) {
            brushTimer = setInterval(brush, 1000/BPS);
        }
	linesRendered = false;
    };

    var brush = function () {
        if (mouseDown) {
            var x = mouseCellX;
            var y = mouseCellY;

            var yStart = Math.max(0, Math.floor(y - brushSize+1));
            var yEnd = Math.min(area.height-1, Math.floor(y + brushSize-1));
            var xStart = Math.max(0, Math.floor(x - brushSize+1));
            var xEnd = Math.min(area.width-1, Math.floor(x + brushSize-1));

            var radius = (brushSize-1)*(brushSize-1);            
            for (var i = yStart; i <= yEnd; i++) {
                for (var j = xStart; j <= xEnd; j++) {
                    if ((x-j)*(x-j) + (y-i)*(y-i) <= radius && (!pieruMode || Math.random() > 0.9)) {
                        var cell = cells[i*area.width + j];
                        switch (currentType) {
                            case EDITOR.TYPE_GROUND:
                            cell.ground = currentProto; 
                            break;
                            case EDITOR.TYPE_NPC:
                            cell.monster = currentProto; 
                            break;
                            case EDITOR.TYPE_ITEM:
                            cell.item = currentProto; 
                            break;
                            case EDITOR.TYPE_ERASER:
                            cell.monster = null;
                            cell.item = null;
                            break;
                        }
			renderCell(cell, j, i);
                    }
                }
            }
        }
    };

    return {
        TYPE_GROUND: 1,
        TYPE_NPC: 2,
        TYPE_ITEM: 3,
        TYPE_ERASER: 4,

        initMap: function(width, height) {
            initArea(parseInt(width), parseInt(height));
            initCells();
        },

        loadMap: function (files) {
            var file = files[0];
            var reader = new FileReader();
            reader.onload = function(event) {
                var mapData = event.target.result;
                var lines = mapData.split('\n');
                var dimensions = lines[0].split(" ");
                var width = parseInt(dimensions[0]);                
                var height = parseInt(dimensions[1]);
                var npcNum = parseInt(lines[1+height]);
                var itemNum = parseInt(lines[2+height+npcNum]);
       
                for (var i = 1; i < height + 1; i++) {
                    for (var j = 0; j < width; j++) {
                        cells[width*(i-1) + j] = {
                            ground: charToProto(lines[i].charAt(j)),
                            monster: null,
                            item: null
                        };
                    }
                }

                for (var i = 2 + height; i < 2 + height + npcNum; i++) {
                    var parts = lines[i].split(";");

                    var name = parts[0];
                    var x = parseInt(parts[1]);
                    var y = parseInt(parts[2]);

                    cells[y*width + x].monster = nameToProto(name);
                }

                for (var i = 3 + height + npcNum; i < 3 + height + npcNum + itemNum; i++) {
                    var parts = lines[i].split(";");

                    var name = parts[0];
                    var x = parseInt(parts[1]);
                    var y = parseInt(parts[2]);

                    cells[y*width + x].item = nameToProto(name);
                }

                initArea(width, height);
            };
            reader.onerror = function(event) {
                alert("Error loading map file");
            };

            reader.readAsText(file, "UTF-8");
        },

        loadProtos: function (files, objectType) {
            var file = files[0];
            var reader = new FileReader();
            reader.onload = function(event) {
                var protoData = event.target.result;
                var lines = protoData.split('\n');
                for (var i = 0; i < lines.length; i++) {
                    var parts = lines[i].split(";");
                    if (parts[0].length == 1) {
                        protos.push({
                                id: idCount++,
                                type: objectType,
                                name: parts[2],
                                keyCharacter: parts[0],
                                color: genColor(idCount)
                                     });
                    } else {
                        protos.push({
                                id: idCount++,
                                type: objectType,
                                name: parts[0],
                                keyCharacter: " ",
                                color: genColor(idCount)
                                    });
                    }
                }
            };
            reader.onerror = function(event) {  
                alert("Error loading prototypes");
            }

            reader.readAsText(file, "UTF-8");
        },

        changeType: function (type) {
            var options = "";
            for (var i = 0; i < protos.length; i++) {
                var proto = protos[i];
                if (proto.type === type)
                    options += '<option value="' + proto.name + '">' + proto.name + '</option>\n';
             }
             $('select').html(options);                
             currentType = type;
        },

        changeSize: function (size) {
            brushSize = parseInt(size);
        },

        changePieruMode: function (mode) {
            pieruMode = mode;    
        },

        changeCurrentProto: function (name) {
            currentProto = nameToProto(name);
        },

        onMouseDown: function(event) {
            mouseDown = true;
        },

        onMouseMove: function(event) {
            mouseCellX = Math.floor(event.offsetX / CELL_SIZE);
            mouseCellY = Math.floor(event.offsetY / CELL_SIZE);
            
            var cell = cells[mouseCellY*area.width + mouseCellX];
            if (cell !== undefined) {
            var npcStr = cell.monster === null ? "" : " NPC: " + cell.monster.name;
            var itemStr = cell.item === null ? "" : " Item: " + cell.item.name;
            $('#info').html("(" + mouseCellX + "," + mouseCellY + ")<br>Ground: " + cell.ground.name + npcStr + itemStr);
            }
        },

        onMouseUp: function(event) {
            mouseDown = false;
        },

        increaseWidth: function() {
            var newCells = [];
            for (var i = 0; i < cells.length; i++) {
                newCells[i + Math.floor(i / area.width)] = cells[i];
            }
            for (var i = area.width; i < (area.width+1)*area.height; i += (area.width+1)) {
                newCells[i] = { ground: nameToProto("grass"),
                                monster: null,
                                item: null};
            }
            cells = newCells;
            initArea(area.width+1, area.height);
        },

        decreaseWidth: function() {
            var newCells = [];
            for (var i = 0; i < cells.length; i++) {
                newCells[i - Math.floor(i / area.width)] = cells[i];
            }
            cells = newCells;
            initArea(area.width-1, area.height);
        },

        decreaseHeight: function() {
            cells = cells.slice(0,cells.length-area.width);
            initArea(area.width, area.height-1);
        },

        increaseHeight: function() {
            var newCells = [];
            for (var i = 0; i < area.width; i++) {
                newCells[i] = { ground: nameToProto("grass"),
                                monster: null,
                                item: null};
            }
            cells = cells.concat(newCells);
            initArea(area.width, area.height+1);
        },

        genMapData: function() {
            var data = "";
            var npcData = "";
            var npcNum = 0;
            var itemData = "";
            var itemNum = 0;
            data += area.width + " " + area.height + "\n";
            for (var i = 0; i < area.height; i++) {
                for (var j = 0; j < area.width; j++) {
                    var cell = cells[i*area.width + j];
                    data += cell.ground.keyCharacter;
                    var npc = cell.monster;
                    if (npc !== null) {
                        npcData += npc.name + ";" + j + ";" + i + ";0;0\n";
                        npcNum++;
                    }
                    var item = cell.item;
                    if (item !== null) {
                        itemData += item.name + ";" + j + ";" + i + "\n";
                        itemNum++;
                    }
                }
                data += "\n";
            }

            data += npcNum + "\n";
            data += npcData;

            data += itemNum + "\n";
            data += itemData.substring(0, itemData.length - 1);
            return data;
        }
    };
}());