package chars;

import items.OwnerType;

import java.awt.Color;
import java.util.HashMap;

import maps.GroundType;
import util.ColorData;
import util.Util;

public class NPCPrototype {

    private static final String FILE_NAME = "npcs.dat";

    private static HashMap<String, NPCPrototype> protos = new HashMap<String, NPCPrototype>();
    private String name;
    private OwnerType type;
    private char symbol;
    private Color color;
    private double maxHp;
    private double maxMp;
    private int speed;
    private double movingPr;
    private int movingR;
    private int sight;
    private GroundType groundType;
    private boolean hostile;
    private int minDmg, maxDmg;
    private boolean rideable;
    private String talk;

    public NPCPrototype(String[] attributes) {
	this.name = attributes[0];
	this.type = OwnerType.stringToOwnerType(attributes[1]);
	String[] colorInfo = attributes[3].split(" ");
	symbol = attributes[2].charAt(0);
	color = ColorData.getColor(colorInfo[0], Integer.parseInt(colorInfo[1]));
	this.maxHp = Integer.parseInt(attributes[4]);
	this.speed = Integer.parseInt(attributes[5]);			// moving speed
	this.movingPr = Double.parseDouble(attributes[6]);		// random movement probability
	this.movingR = Integer.parseInt(attributes[7]);			// moving radius from orig point
	this.sight = Integer.parseInt(attributes[8]);			// how far sees
	this.groundType = GroundType.get(attributes[9]);
	this.hostile = Boolean.parseBoolean(attributes[10]);
	this.minDmg = Integer.parseInt(attributes[11]);
	this.maxDmg = Integer.parseInt(attributes[12]);
	this.rideable = Boolean.parseBoolean(attributes[13]);
	if (attributes.length > 14)
	    this.talk = attributes[14];
	else
	    this.talk = null;
    }

    public static void loadPrototypes() {
	for (String[] attributes : Util.getPrototypeAttributes(FILE_NAME, ";")) {
	    protos.put(attributes[0], new NPCPrototype(attributes));
	}
    }

    public static NPCPrototype getPrototype(String name) {
	return protos.get(name);
    }

    public String getName() {
	return this.name;
    }

    public char getSymbol() {
	return symbol;
    }
    
    public Color getColor() {
	return color;
    }

    public double getMaxHp() {
	return this.maxHp;
    }

    public double getMaxMp() {
	return this.maxMp;
    }

    public int getSpeed() {
	return this.speed;
    }

    public double getMovingPr() {
	return this.movingPr;
    }

    public int getMovingR() {
	return this.movingR;
    }

    public int getSight() {
	return this.sight;
    }
    
    public GroundType getGroundType() {
	return groundType;
    }

    public boolean isHostile() {
	return this.hostile;
    }

    public int getMinDmg() {
	return minDmg;
    }

    public int getMaxDmg() {
	return maxDmg;
    }

    public OwnerType getType() {
	return type;
    }
    
    public boolean isRideable() {
	return rideable;
    }

    public String getTalk() {
	return talk;
    }
}
