package items;

import java.awt.Color;

import maps.Cell;
import models.GameModel;
import util.ColorData;
import util.Util;
import chars.Player;

public class Item {

    private static final double RANDOM_FACTOR = 0.2;

    private String name;
    private char symbol;
    private String cName;
    private int cBrightness;
    private boolean eatable;
    private boolean makesSick;
    private int eatHp;
    private int eatMp;
    private WieldSpot wieldSpot;
    private int attValue;
    private int defValue;
    private int mgcValue;
    private double effVariance;
    private double weight;
    private boolean wielded;
    private Player player;
    private Cell cell;

    public Item(String name, Player player) {
	this(name, null, player);
    }

    public Item(String name) {
	this(name, null, null);
    }

    private Item(String name, Cell cell, Player player) {
	ItemPrototype proto = ItemPrototype.getPrototype(name);
	this.name = proto.getName();
	symbol = proto.getSymbol();
	cName = proto.getColorName();
	cBrightness = proto.getColorBrightness();
	eatable = proto.isEatable();
	makesSick = proto.makesSick();
	eatHp = proto.getEatHp();
	eatMp = proto.getEatMp();
	wieldSpot = proto.getWieldSpot();
	attValue = Util.random(proto.getAtt(), RANDOM_FACTOR);
	defValue = Util.random(proto.getDef(), RANDOM_FACTOR);
	mgcValue = Util.random(proto.getMgc(), RANDOM_FACTOR);
	effVariance = proto.getEffVariance();
	weight = proto.getWeight();
	wielded = false;
	this.cell = cell;
	this.player = player;

	Util.log("Creating item: " + name);
    }
/*
    public void draw(Graphics2D g) {
	if (cell != null) {
	    GameModel gameModel = cell.getMap().getGameModel();
	    int drawX = GameView.getDrawX(gameModel, getCell().getX());
	    int drawY = GameView.getDrawY(gameModel, getCell().getY());
	    if (gameModel.getTimeOfDay() == GameModel.NIGHT)
		g.setColor(ColorData.getNightColor(colorName, colorBrightness)); //TODO clear?
	    else
		g.setColor(ColorData.getColor(colorName, colorBrightness));
	    View.drawCharAt(g, symbol, drawX, drawY);
	}
    }*/

    public String use() {
	if (this.isEatable()) {
	    player.increaseHp(this.eatHp);
	    player.increaseMp(this.eatMp);
	    player.getItems().remove(this);
	    return "Slurps! That " + this.getName() + " was delicious!";
	}
	return "";
    }

    public void wield() {
	if (wieldSpot != WieldSpot.WIELD_NONE) {
	    Item wieldedItem = wieldSpot.getWieldedItem();
	    if (wieldedItem != this) {
		if (wieldedItem != null)
		    wieldedItem.setWielded(false);
		this.setWielded(true);
		wieldSpot.setWieldedItem(this);
	    } else {
		this.setWielded(false);
		wieldSpot.setWieldedItem(null);
	    }
	}
    }

    public void drop() {
	if (this.isWielded()) {
	    this.getWieldSpot().setWieldedItem(null);
	    setWielded(false);
	}
	cell = player.getCell();
	player.getCell().getItems().add(this);
	player.getItems().remove(this);
	player.updateItemsWeight();
	player = null;
    }

    public String getAttrString() {
	String attrString = "";
	attrString += "(" + this.getWeight() + " kgs";
	if (this.getAttMax() > 0)
	    attrString += ", att " + getValueString(getAttMin(), getAttMax());
	if (this.getMgcMax() > 0)
	    attrString += ", mgc " + getValueString(getMgcMin(), getMgcMax());
	if (this.getDefMax() > 0)
	    attrString += ", def " + getValueString(getDefMin(), getDefMax());
	attrString += ")";

	return attrString;
    }
    
    private String getValueString(int min, int max) {
	if (min == max)
	    return Integer.toString(min);
	else
	    return min + "-" + max;
    }

    public String getName() {
	return this.name;
    }
    
    public char getSymbol() {
	return symbol;
    }
    
    public String getColorName() {
	return cName;
    }
    
    public int getColorBrightness() {
	return cBrightness;
    }
    
    public Color getInstantColor() {
	if (cell.getMap().getGameModel().getTimeOfDay() == GameModel.NIGHT)
	    return ColorData.getNightColor(cName, cBrightness);
	else
	    return ColorData.getColor(cName, cBrightness);
    }

    public boolean isEatable() {
	return this.eatable;
    }

    public boolean isCombatItem() {
	return attValue > 0 || defValue > 0 || mgcValue > 0 ? true : false;
    }

    public boolean makesSick() {
	return this.makesSick;
    }

    public int getEatHp() {
	return this.eatHp;
    }

    public void setCell(Cell cell) {
	this.cell = cell;
    }

    public void setPlayer(Player player) {
	this.player = player;
    }

    public int getEatMgc() {
	return this.eatMp;
    }

    public WieldSpot getWieldSpot() {
	return this.wieldSpot;
    }

    public int getAttMax() {
	return this.attValue;
    }

    public int getAttMin() {
	return (int) (this.attValue * this.effVariance);
    }

    public int getDefMax() {
	return this.defValue;
    }

    public int getDefMin() {
	return (int) (this.defValue * this.effVariance);
    }

    public int getMgcMax() {
	return this.mgcValue;
    }

    public int getMgcMin() {
	return (int) (this.mgcValue * this.effVariance);
    }

    public double getWeight() {
	return weight;
    }

    public void setWielded(boolean wielded) {
	this.wielded = wielded;
    }

    public boolean isWielded() {
	return wielded;
    }

    public Cell getCell() {
	return cell;
    }
}
