package items;

import java.util.ArrayList;

import lists.JList;
import lists.JListObject;
import views.View;

public class ItemJList extends JList {

    ArrayList<JListObject> wieldedItems;

    private int combatItemsIndex;
    private int foodItemsIndex;
    private int otherItemsIndex;

    private int objectsPerPage;
    private int separatorLines = 0;

    public ItemJList(int objectsDisplayed) {
	super(objectsDisplayed);
	objectsPerPage = objectsDisplayed;
    }

    public void update() {
	wieldedItems = new ArrayList<JListObject>();
	ArrayList<JListObject> combatItems = new ArrayList<JListObject>();
	ArrayList<JListObject> foodItems = new ArrayList<JListObject>();
	ArrayList<JListObject> otherItems = new ArrayList<JListObject>();
	for (JListObject o : objects) {
	    Item i = (Item) o.getObject();

	    if (i.isWielded()) {
		wieldedItems.add(o);
	    } else if (i.isCombatItem()) {
		combatItems.add(o);
	    } else if (i.isEatable()) {
		foodItems.add(o);
	    } else {
		otherItems.add(o);
	    }
	}

	correctLetters(wieldedItems);

	combatItems = alphabeticalOrder(combatItems);
	foodItems = alphabeticalOrder(foodItems);
	otherItems = alphabeticalOrder(otherItems);

	combatItemsIndex = wieldedItems.size();
	foodItemsIndex = combatItemsIndex + combatItems.size();
	otherItemsIndex = foodItemsIndex + foodItems.size();

	separatorLines = (isFoodsSeparator() ? 1 : 0)
		- (isOthersSeparator() ? 1 : 0);
	objectsPerPage = View.HEIGHT - 19 - separatorLines;
	objectsDisplayed = objectsPerPage + wieldedItems.size();

	ArrayList<JListObject> newObjects = new ArrayList<JListObject>();
	newObjects.addAll(combatItems);
	newObjects.addAll(foodItems);
	newObjects.addAll(otherItems);
	newObjects.addAll(firstVisibleIndex, wieldedItems);
	objects = newObjects;

	super.update();
    }

    public boolean isFoodsSeparator() {
	return foodItemsIndex > firstVisibleIndex
		&& firstVisibleIndex + objectsPerPage > foodItemsIndex
		&& !(foodItemsIndex == otherItemsIndex);
    }

    public boolean isOthersSeparator() {
	return otherItemsIndex > firstVisibleIndex
		&& firstVisibleIndex + objectsPerPage > otherItemsIndex;
    }

    public void nextPage() {
	setFirstVisibleIndex(firstVisibleIndex + objectsPerPage > objects
		.size() - 1 ? firstVisibleIndex : firstVisibleIndex
		+ objectsPerPage);
	update();
    }

    public void previousPage() {
	setFirstVisibleIndex(Math.max(0, firstVisibleIndex - objectsPerPage
		- separatorLines));
	update();
    }

    public int getPageNumber() {
	return (firstVisibleIndex - wieldedItems.size()) / objectsPerPage + 1;
    }

    public int getNumberOfPages() {
	if (objects.size() - wieldedItems.size() == objectsPerPage)
	    return 1;
	return (objects.size() - wieldedItems.size())
		/ objectsPerPage
		+ ((objects.size() - wieldedItems.size()) % objectsPerPage == 0 ? 0
			: 1);
    }

    public int getFirstVisibleIndex() {
	return firstVisibleIndex;
    }

    private void correctLetters(ArrayList<JListObject> list) {
	int listSize = list.size();
	if (listSize > 1) {
	    ArrayList<JListObject> newList = new ArrayList<JListObject>();

	    for (int i = 0; i < listSize; i++) {
		JListObject candidate = list.get(0);
		for (int j = 0; j < list.size(); j++) {
		    if (((Item) list.get(j).getObject()).getWieldSpot()
			    .getType() < ((Item) candidate.getObject())
			    .getWieldSpot().getType()) {
			candidate = list.get(j);
		    }
		}
		newList.add(candidate);
		list.remove(candidate);
	    }
	    for (JListObject o : newList) {
		list.add(o);
	    }
	}
    }

    private static ArrayList<JListObject> alphabeticalOrder(
	    ArrayList<JListObject> list) {
	if (list.size() < 2)
	    return list;

	ArrayList<JListObject> newList = new ArrayList<JListObject>();

	int listSize = list.size();
	for (int i = 0; i < listSize; i++) {
	    JListObject candidate = list.get(0);
	    for (int j = 0; j < list.size(); j++) {
		if (((Item) list.get(j).getObject()).getName()
			.compareToIgnoreCase(
				((Item) candidate.getObject()).getName()) < 0) {
		    candidate = list.get(j);
		}
	    }
	    newList.add(candidate);
	    list.remove(candidate);
	}
	return newList;
    }

    public int getCombatItemsIndex() {
	return combatItemsIndex;
    }

    public int getFoodItemsIndex() {
	return foodItemsIndex;
    }

    public int getOtherItemsIndex() {
	return otherItemsIndex;
    }

    public ArrayList<JListObject> getWieldedItems() {
	return wieldedItems;
    }
}
