package items;

import java.util.ArrayList;
import java.util.HashMap;

import util.Util;
import chars.NPC;

public class ItemPrototype {

    private static final String FILE_NAME = "items.dat";

    private static HashMap<String, ItemPrototype> protos = new HashMap<String, ItemPrototype>();

    private String name;
    private char symbol;
    private String cName;
    private int cBrightness;
    private boolean eatable;
    private boolean makesSick;
    private int eatHp;
    private int eatMp;
    private WieldSpot wieldSpot;
    private int att;
    private int def;
    private int mgc;
    private double effVariance;
    private double weight;

    public ItemPrototype(String[] attributes) {
	this.name = attributes[0];
	addToItemProtosLists(attributes[1].split(","));
	symbol = attributes[2].charAt(0);
	String[] colorInfo = attributes[3].split(" ");
	cName = colorInfo[0];
	cBrightness = Integer.parseInt(colorInfo[1]);
	this.eatHp = Integer.parseInt(attributes[4]);
	this.eatMp = Integer.parseInt(attributes[5]);
	this.wieldSpot = WieldSpot.stringToWieldSpot(attributes[6]);
	this.att = Integer.parseInt(attributes[7]);
	this.def = Integer.parseInt(attributes[8]);
	this.mgc = Integer.parseInt(attributes[9]);
	this.effVariance = Double.parseDouble(attributes[10]);
	this.weight = Double.parseDouble(attributes[11]);

	this.eatable = (this.eatHp != 0) || (this.eatMp != 0);
	this.makesSick = this.eatHp < 0;
    }

    private void addToItemProtosLists(String[] types) {
	for (int i = 0; i < types.length; i++) {
	    OwnerType ownerType = OwnerType.stringToOwnerType(types[i]);
	    ownerType.getItems().add(this);
	}
    }

    public static void loadPrototypes() {
	for (String[] attributes : Util
		.getPrototypeAttributes(FILE_NAME, ";")) {
	    protos.put(attributes[0], new ItemPrototype(attributes));
	}
    }

    public static Item generateDroppedItem(NPC npc) {
	ArrayList<ItemPrototype> targetItems = npc.getType().getItems();
	int randomIndex = Util.randInt(0, targetItems.size() - 1);
	return new Item(targetItems.get(randomIndex).getName());
    }

    public static ItemPrototype getPrototype(String name) {
	return protos.get(name);
    }

    public String getName() {
	return this.name;
    }
    
    public char getSymbol() {
	return symbol;
    }
    
    public String getColorName() {
	return cName;
    }
    
    public int getColorBrightness() {
	return cBrightness;
    }

    public boolean isEatable() {
	return this.eatable;
    }

    public boolean makesSick() {
	return this.makesSick;
    }

    public int getEatHp() {
	return this.eatHp;
    }

    public int getEatMp() {
	return this.eatMp;
    }

    public WieldSpot getWieldSpot() {
	return this.wieldSpot;
    }

    public int getAtt() {
	return this.att;
    }

    public int getDef() {
	return this.def;
    }

    public int getMgc() {
	return this.mgc;
    }

    public double getEffVariance() {
	return this.effVariance;
    }

    public double getWeight() {
	return this.weight;
    }

}
