package lists;

import items.Item;

import java.util.ArrayList;

public class ItemList {

    private int itemsPerPage;

    private ArrayList<ItemPage> pages;

    private ArrayList<Item> wieldedItems;
    private ArrayList<Item> combatItems;
    private ArrayList<Item> foodItems;
    private ArrayList<Item> otherItems;

    private ItemPage currentPage, lastPage;
    private int oldCurrentPageIndex = 0;

    public ItemList(int itemsPerPage) {
	this.itemsPerPage = itemsPerPage;

	pages = new ArrayList<ItemPage>();
	wieldedItems = new ArrayList<Item>();
	combatItems = new ArrayList<Item>();
	foodItems = new ArrayList<Item>();
	otherItems = new ArrayList<Item>();
	clear();
    }

    public void clear() {
	wieldedItems.clear();
	combatItems.clear();
	foodItems.clear();
	otherItems.clear();

	clearPages();
    }

    private void clearPages() {
	pages.clear();
	pages.add(new ItemPage(itemsPerPage));
	currentPage = pages.get(0);
	lastPage = currentPage;
    }

    public int size() {
	return wieldedItems.size() + combatItems.size() + foodItems.size()
		+ otherItems.size();
    }

    public void addItem(Item item) {
	if (item.isWielded()) {
	    wieldedItems.add(item);
	} else if (item.isCombatItem()) {
	    combatItems.add(item);
	} else if (item.isEatable()) {
	    foodItems.add(item);
	} else {
	    otherItems.add(item);
	}
    }

    public void removeItem(Item item) {
	if (item.isWielded()) {
	    wieldedItems.remove(item);
	} else if (item.isCombatItem()) {
	    combatItems.remove(item);
	} else if (item.isEatable()) {
	    foodItems.remove(item);
	} else {
	    otherItems.remove(item);
	}
    }

    public int getNumberOfPages() {
	return pages.size();
    }

    public int getCurrentPageNumber() {
	return pages.indexOf(currentPage) + 1;
    }

    public void update(ArrayList<Item> items) {
	oldCurrentPageIndex = pages.indexOf(currentPage);
	clear();

	for (Item i : items) {
	    addItem(i);
	}

	wieldedItems = wieldSpotOrder(wieldedItems);
	combatItems = alphabeticalOrder(combatItems);
	foodItems = alphabeticalOrder(foodItems);
	otherItems = alphabeticalOrder(otherItems);

	addListToPage(combatItems);
	addListToPage(foodItems);
	addListToPage(otherItems);
	for (ItemPage p : pages) {
	    p.addToBottom(wieldedItems);
	}

	for (int i = 0; i < pages.size(); i++) {
	    if (oldCurrentPageIndex - i < pages.size()
		    && oldCurrentPageIndex - i >= 0) {
		currentPage = pages.get(oldCurrentPageIndex - i);
		i = pages.size();
	    }
	}
    }

    private void addListToPage(ArrayList<Item> list) {
	for (ItemPage page : pages) {
	    page.setFirstLetterIndex(wieldedItems.size());
	}

	for (int i = 0; i < list.size(); i++) {
	    if (!lastPage.isFull()) {
		lastPage.addItem(list.get(i));
	    } else {
		addNewPage();
		i--;
	    }
	}
	if (!lastPage.isFull() && list.size() > 0)
	    lastPage.addSeparator();
    }

    private void addNewPage() {
	ItemPage newPage = new ItemPage(itemsPerPage);
	newPage.setFirstLetterIndex(wieldedItems.size());
	pages.add(newPage);
	lastPage = newPage;
    }

    private static ArrayList<Item> wieldSpotOrder(ArrayList<Item> list) {
	if (list.size() < 2)
	    return list;

	ArrayList<Item> newList = new ArrayList<Item>();

	int listSize = list.size();
	for (int i = 0; i < listSize; i++) {
	    Item candidate = list.get(0);
	    for (int j = 0; j < list.size(); j++) {
		if (list.get(j).getWieldSpot().getType() < candidate
			.getWieldSpot().getType()) {
		    candidate = list.get(j);
		}
	    }
	    newList.add(candidate);
	    list.remove(candidate);
	}
	return newList;
    }

    private static ArrayList<Item> alphabeticalOrder(ArrayList<Item> list) {
	if (list.size() < 2)
	    return list;

	ArrayList<Item> newList = new ArrayList<Item>();

	int listSize = list.size();
	for (int i = 0; i < listSize; i++) {
	    Item candidate = list.get(0);
	    for (int j = 0; j < list.size(); j++) {
		if (list.get(j).getName()
			.compareToIgnoreCase(candidate.getName()) < 0) {
		    candidate = list.get(j);
		}
	    }
	    newList.add(candidate);
	    list.remove(candidate);
	}
	return newList;
    }

    public ItemPage getCurrentPage() {
	return currentPage;
    }

    public void nextPage() {
	if (currentPage != lastPage) {
	    currentPage = pages.get(pages.indexOf(currentPage) + 1);
	}
    }

    public void prevPage() {
	if (pages.indexOf(currentPage) != 0) {
	    currentPage = pages.get(pages.indexOf(currentPage) - 1);
	}
    }

    public void setCurrentPage(ItemPage currentPage) {
	this.currentPage = currentPage;
    }

    public ArrayList<ItemPage> getPages() {
	return pages;
    }

    public ArrayList<Item> getWieldedItems() {
	return wieldedItems;
    }

}
