package lists;

import java.util.ArrayList;

import util.Util;

public class JList {

    private static final String alphabet = "abcdefghijklmnopqrstuvwxyz";
    protected ArrayList<JListObject> objects;
    protected ArrayList<JListObject> visibleObjects;
    protected int firstVisibleIndex = 0;
    protected int objectsDisplayed;

    public JList(int objectsDisplayed) {
	this.objectsDisplayed = Math.min(objectsDisplayed, alphabet.length());
	objects = new ArrayList<JListObject>();
	visibleObjects = new ArrayList<JListObject>();
    }

    public JList(ArrayList<JListObject> objects, int objectsDisplayed) {
	this.objectsDisplayed = Math.min(objectsDisplayed, alphabet.length());
	this.objects = objects;
    }

    public void addObject(JListObject object) {
	objects.add(object);
	update();
    }

    public void removeObject(JListObject object) {
	objects.remove(object);
	update();
    }

    public void clear() {
	objects.clear();
	update();
    }

    public int size() {
	return objects.size();
    }

    public void setFirstVisibleIndex(int firstVisibleIndex) {
	if (firstVisibleIndex >= 0 && firstVisibleIndex < objects.size()) {
	    this.firstVisibleIndex = firstVisibleIndex;
	    update();
	} else {
	    Util.logError("Can't set first visible index out of bounds in JList.");
	}
    }

    public void update() {
	visibleObjects.clear();
	for (int i = firstVisibleIndex; i < objects.size()
		&& i - firstVisibleIndex < objectsDisplayed; i++) {
	    visibleObjects.add(objects.get(i));
	}

	int i = 0;
	for (JListObject o : visibleObjects) {
	    o.setLetter(alphabet.charAt(i));
	    i++;
	}
    }

    public void nextPage() {
	setFirstVisibleIndex(firstVisibleIndex + objectsDisplayed > objects
		.size() - 1 ? firstVisibleIndex : firstVisibleIndex
		+ objectsDisplayed);
	update();
    }

    public void previousPage() {
	setFirstVisibleIndex(Math.max(0, firstVisibleIndex - objectsDisplayed));
	update();
    }

    public ArrayList<JListObject> getVisibleObjects() {
	return visibleObjects;
    }

    public int getPageNumber() {
	return firstVisibleIndex / objectsDisplayed + 1;
    }

    public int getNumberOfPages() {
	return objects.size() / objectsDisplayed + 1;
    }

    public int getObjectsDisplayed() {
	return objectsDisplayed;
    }

    public ArrayList<String> getListAsStrings() {
	ArrayList<String> strings = new ArrayList<String>();
	for (JListObject o : visibleObjects) {
	    strings.add(o.getLetter() + ") " + o.getName());
	}

	return strings;
    }
}
