package maps;

import items.Item;

import java.awt.Color;
import java.util.ArrayList;

import models.GameModel;
import spells.Spell;
import util.ColorData;
import util.Dir;
import util.Util;
import chars.Char;
import chars.Player;

public class Cell {

    public final static int UNKNOWN = 1;
    public final static int KNOWN = 2;
    public final static int CURRENTLY_SEEN = 3;

    public final static int BURNING_DAMAGE = 5;

    private Map map;
    private Char character;
    private GroundPrototype groundProto;
    private String colorName;
    private int colorBrightness;
    private ArrayList<Item> items = new ArrayList<Item>();
    private ArrayList<Spell> spells = new ArrayList<Spell>();
    private int x, y;
    private int visibility = UNKNOWN;
    private int hp;
    private Link link;
    private boolean isBurning;
    /* path finding related variables */
    private boolean visited;
    private Dir prevDir;

    public Cell(Map map, char keyChar, int x, int y) {
	this.map = map;
	this.x = x;
	this.y = y;
	groundProto = GroundPrototype.getProto(keyChar);
	colorName = groundProto.getColorName();
	colorBrightness = groundProto.getColorBrightness();
	hp = groundProto.getHp();
	isBurning = false;
    }
/*
    public void draw(Graphics2D g) {
	if (visibility == UNKNOWN)
	    return;

	if (visibility == KNOWN) {
	    drawGround(g);
	    return;
	}

	if (character != null) {
	    character.draw(g);
	    return;
	}

	if (items.size() > 0) {
	    items.get(items.size() - 1).draw(g);
	    return;
	}

	drawGround(g);
    }
    private void drawGround(Graphics2D g) {
	GameModel gameModel = map.getGameModel();
	int drawX = GameView.getDrawX(gameModel, getX());
	int drawY = GameView.getDrawY(gameModel, getY());
	int timeOfDay = gameModel.getTimeOfDay();
	boolean known = (visibility == KNOWN);
	switch (timeOfDay) {
	case GameModel.DAY:
	    g.setColor(ColorData.getDayColor(colorName, colorBrightness, known));
	    break;
	case GameModel.MORNING:
	case GameModel.EVENING:
	    g.setColor(ColorData.getEveningColor(colorName, colorBrightness, known));
	    break;
	case GameModel.NIGHT:
	    g.setColor(ColorData.getNightColor(colorName, colorBrightness, known));
	    break;
	}

	View.drawCharAt(g, groundProto.getSymbol(), drawX, drawY);
    }
*/
    
    public Color getInstantColor() {
	GameModel model = map.getGameModel();
	boolean known = (visibility == KNOWN);
	switch (model.getTimeOfDay()) {
	case GameModel.DAY:
	    return ColorData.getDayColor(colorName, colorBrightness, known);
	case GameModel.MORNING:
	case GameModel.EVENING:
	    return ColorData.getEveningColor(colorName, colorBrightness, known);
	case GameModel.NIGHT:
	    return ColorData.getNightColor(colorName, colorBrightness, known);
	default:
	    return null;
	}
    }
    
    public String getColorName() {
	return colorName;
    }
    
    public int getColorBrightness() {
	return colorBrightness;
    }
    
    public char getSymbol() {
	return groundProto.getSymbol();
    }

    public void setChar(Char character) {
	this.character = character;
    }

    public Char getChar() {
	return character;
    }

    public GroundPrototype getGroundProto() {
	return groundProto;
    }

    public boolean isFree() {
	return character == null;
    }

    public int getX() {
	return x;
    }

    public int getY() {
	return y;
    }

    public int getVisibility() {
	return visibility;
    }

    public void setVisibility(int visibility) {
	this.visibility = visibility;
    }

    public Map getMap() {
	return map;
    }

    public Cell getAdjacentCell(Dir dir) {
	int adjX = x + dir.getDeltaX();
	int adjY = y + dir.getDeltaY();
	if (adjX >= 0 && adjX < map.getCells().length && adjY >= 0 && adjY < map.getCells()[0].length)
	    return map.getCells()[adjX][adjY];
	return null;
    }

    public ArrayList<Cell> getAdjacentCells() {
	ArrayList<Cell> adjacentCells = new ArrayList<Cell>();
	for (Dir dir : Dir.DIRS) {
	    if (dir != Dir.NONE) {
		Cell adj = this.getAdjacentCell(dir);
		if (adj != null)
		    adjacentCells.add(adj);
	    }
	}
	return adjacentCells;
    }

    public void addItem(Item item) {
	items.add(item);
	item.setCell(this);
    }

    public ArrayList<Item> getItems() {
	return items;
    }
    
    public ArrayList<Spell> getSpells() {
	return spells;
    }

    public Player getPlayer() {
	return (Player) character;
    }

    public Link getLink() {
	return link;
    }

    public void setLink(Link newLink) {
	link = newLink;
    }

    public boolean burning() {
	return isBurning;
    }

    public boolean isVisited() {
	return visited;
    }

    public void setVisited(boolean visited) {
	this.visited = visited;
    }

    public Dir getPrevDir() {
	return prevDir;
    }

    public void setPrevDir(Dir prevDir) {
	this.prevDir = prevDir;
    }

    public void burn() {
	hp -= groundProto.getBurningFactor() * map.getTimeLeap() * Util.random(BURNING_DAMAGE, 1);
	ArrayList<Cell> cellsNextTo = getAdjacentCells();
	for (Cell c : cellsNextTo) {
	    if (!c.isBurning)
		c.tryToSetOnFire(false);
	}
	if (hp <= groundProto.getHp() * 0.1) {
	    isBurning = false;
	    groundProto = GroundPrototype.getProto("burnt " + groundProto.getType().getName());
	    if (hp <= 0) {
		hp = 0;
		groundProto = GroundPrototype.getProto("burnt ground");
	    }
	    colorName = groundProto.getColorName();
	    colorBrightness = groundProto.getColorBrightness();
	} else {
	    int burningCellsNear = 0;
	    for (Cell c : cellsNextTo) {
		if (c.isBurning)
		    burningCellsNear++;
	    }
	    if (burningCellsNear < 2 && groundProto.getBurningFactor() * map.getTimeLeap() / 5 > Math.random()) {
		isBurning = false;
		colorName = groundProto.getColorName();
		colorBrightness = groundProto.getColorBrightness();
		if (hp < groundProto.getHp() / 3)
		    colorBrightness = (colorBrightness - 1 < 0 ? 0 : colorBrightness - 1);
		}
	    }
	}

    public void tryToSetOnFire(boolean purposely) {
	double burningFactor = groundProto.getBurningFactor() * map.getTimeLeap();
	if (!purposely)
	    burningFactor = burningFactor * 0.05;
	if (burningFactor > Math.random()) {
	    isBurning = true;
	    colorName = "orange";
	    colorBrightness = 3;
	    map.toBurningList(this);
	}
    }

    public void decreaseHp(int dmg) {
	hp -= dmg;
	if (hp <= 0) {
	    hp = 0;
	    groundProto = GroundPrototype.getProto("pile of wreck");
	    colorName = groundProto.getColorName();
	    colorBrightness = groundProto.getColorBrightness();
	} else if (hp < groundProto.getHp() / 3) {
	    colorBrightness = (colorBrightness - 1 < 0 ? 0 : colorBrightness - 1);
	}
    }
}
