package models;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;

import ui.Window;
import util.Util;
import views.HelpView;
import views.View;
import controllers.HelpController;

public class HelpModel extends Model {

    private static final String FILENAME = "help.txt";
    public static final int PAGE_LENGTH = View.HEIGHT - 5;

    private ArrayList<String> lines;
    private int firstLine = 0;

    public HelpModel(Window window) {
	super(window);
	this.view = new HelpView(this);
	this.controller = new HelpController(this);
	this.lines = new ArrayList<String>();
	loadHelp();
    }

    private void loadHelp() {
	try {
	    BufferedReader reader = new BufferedReader(new InputStreamReader(
		    new FileInputStream(FILENAME), "UTF-8"));

	    String line;
	    while ((line = reader.readLine()) != null) {
		lines.add(line);
	    }
	    reader.close();
	} catch (Exception e) {
	    Util.logError("Couldn't read " + FILENAME + ".");
	}
    }
    
    public int getFirstLine() {
	return firstLine;
    }
    
    public ArrayList<String> getLines() {
	return lines;
    }

    public void previousPage() {
	firstLine = Math.max(0, firstLine - PAGE_LENGTH);
    }

    public void nextPage() {
	if (firstLine + PAGE_LENGTH < lines.size())
	    firstLine += PAGE_LENGTH;
    }

}
