package spells;

import java.awt.Graphics2D;

import models.GameModel;
import ui.Jorma;
import util.Dir;
import chars.Char;

public abstract class Spell {

    protected Char caster;
    protected String name;
    protected int cost;
    protected int maxDuration;
    protected int duration;
    protected Dir direction;
    protected boolean directed;
    protected boolean learned;
    protected int level;

    protected Spell(Char caster, String name, int cost, int maxDuration, boolean directed, int level) {
	this.caster = caster;
	this.name = name;
	this.cost = cost;
	this.maxDuration = maxDuration;
	this.directed = directed;
	this.learned = true;
	this.level = level;
    }

    public void cast(Dir direction) {
	this.direction = direction;
	this.duration = maxDuration;
	GameModel gameModel = ((GameModel) Jorma.MODEL_GAME);
	gameModel.addMessage("You cast spell " + this.getName() + "!");

	caster.decreaseMp(this.getCost());
	caster.getMap().addAffectingSpell(this);

	doStartEffect();
    }

    public void decreaseDuration() {
	if (duration > 0) {
	    duration--;
	    if (duration == 0) {
		doEndEffect();
	    }
	}
    }
    
    protected abstract void doStartEffect();
    
    public abstract void draw(Graphics2D g);
    
    protected abstract void doEndEffect();

    public String getName() {
	return name;
    }

    public int getCost() {
	return cost;
    }

    public int getDuration() {
	return duration;
    }

    public int getMaxDuration() {
	return maxDuration;
    }

    public boolean isDirected() {
	return directed;
    }

    public boolean isLearned() {
	return learned;
    }
}
