package ui;

import items.ItemPrototype;
import maps.GroundPrototype;
import models.GameModel;
import models.HelpModel;
import models.HighscoresModel;
import models.InventoryModel;
import models.MenuModel;
import models.Model;
import models.SkillsModel;
import models.SpellsModel;
import util.ColorData;
import chars.NPCPrototype;

public class Jorma {

    public static Model MODEL_GAME;
    public static Model MODEL_INVENTORY;
    public static Model MODEL_HIGHSCORES;
    public static Model MODEL_MENU;
    public static Model MODEL_SPELLS;
    public static Model MODEL_HELP;
    public static Model MODEL_SKILLS;

    private Window window;
    private Screen screen;
    private Model currentModel;

    public static void main(String[] args) {
	new Jorma();
    }

    public Jorma() {
	initWindow();
	ColorData.initColors();
	loadPrototypes();
	initModels();

	currentModel = MODEL_GAME;
	((GameModel) currentModel).newTurn();
    }

    private void initWindow() {
	window = new Window(this);
	screen = new Screen(this);
	window.add(screen);
    }

    private void initModels() {
	MODEL_GAME = new GameModel(window);
	MODEL_INVENTORY = new InventoryModel(window);
	MODEL_HIGHSCORES = new HighscoresModel(window);
	MODEL_MENU = new MenuModel(window);
	MODEL_SPELLS = new SpellsModel(window);
	MODEL_HELP = new HelpModel(window);
	MODEL_SKILLS = new SkillsModel(window);
    }

    private void loadPrototypes() {
	GroundPrototype.loadPrototypes();
	NPCPrototype.loadPrototypes();
	ItemPrototype.loadPrototypes();
    }

    public Window getWindow() {
	return window;
    }

    public Model getCurrentModel() {
	return currentModel;
    }

    public void setCurrentModel(Model model) {
	this.currentModel = model;
    }

}