package util;

import java.awt.Color;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;

public class ColorData {

    private static HashMap<String, Color[]> COLOR_MAP = new HashMap<String, Color[]>();
    private static HashMap<String, Color[]> COLORS_DAY_SEEN = new HashMap<String, Color[]>();
    private static HashMap<String, Color[]> COLORS_DAY_UNSEEN = new HashMap<String, Color[]>();
    private static HashMap<String, Color[]> COLORS_EVENING_SEEN = new HashMap<String, Color[]>();
    private static HashMap<String, Color[]> COLORS_EVENING_UNSEEN = new HashMap<String, Color[]>();
    private static HashMap<String, Color[]> COLORS_NIGHT_SEEN = new HashMap<String, Color[]>();
    private static HashMap<String, Color[]> COLORS_NIGHT_UNSEEN = new HashMap<String, Color[]>();
    private static String COLORS_FILENAME = "colors.dat";
    private static int BRIGHTNESS_DEGREES = 6;

    public static void initColors() {
	ArrayList<String> colorNames = new ArrayList<String>();
	int lineCount = 1;
	try {
	    BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(COLORS_FILENAME), "UTF-8"));
	    String line;
	    while ((line = reader.readLine()) != null) {
		String[] parts = line.split(";");
		String name = parts[0];
		String[] colorData = parts[1].split(",");
		Color[] colors = new Color[BRIGHTNESS_DEGREES];
		int i = 0;
		for (String data : colorData) {
		    String numbers[] = data.split(" ");
		    colors[i] = new Color(Integer.parseInt(numbers[0]), Integer.parseInt(numbers[1]), Integer.parseInt(numbers[2]));
		    i++;
		}
		COLOR_MAP.put(name, colors);
		colorNames.add(name);
		lineCount++;
	    }
	    reader.close();
	} catch (Exception e) {
	    Util.logError("Couldn't read " + COLORS_FILENAME + " at line " + lineCount + ".");
	}

	for (String name : colorNames) {
	    Color[] colors = COLOR_MAP.get(name);
	    COLORS_DAY_SEEN.put(name, generateDaySeen(colors));
	    COLORS_DAY_UNSEEN.put(name, generateDayUnseen(colors));
	    COLORS_EVENING_SEEN.put(name, COLORS_DAY_SEEN.get(name));
	    COLORS_EVENING_UNSEEN.put(name, generateEveningUnseen(colors));
	    COLORS_NIGHT_SEEN.put(name, generateNightSeen(colors));
	    COLORS_NIGHT_UNSEEN.put(name, generateNightUnseen(colors));
	}
    }
    
    private static Color[] generateDaySeen(Color[] colors) {
	Color[] newColors = new Color[colors.length];
	for (int i = 0; i < colors.length; i++) {
	    newColors[i] = i + 1 < colors.length ? colors[i + 1] : COLOR_MAP.get("white")[0];
	}
	return newColors;
    }

    private static Color[] generateDayUnseen(Color[] colors) {
	Color[] newColors = new Color[colors.length];
	for (int i = 0; i < colors.length; i++) {
	    newColors[i] = mixColors(colors[i], i - 1 < 0 ? COLOR_MAP.get("black")[i] : colors[i - 1]);
	}
	return newColors;
    }

    private static Color[] generateEveningUnseen(Color[] colors) {
	Color[] newColors = new Color[colors.length];
	for (int i = 0; i < colors.length; i++) {
	    newColors[i] = mixColors(colors[i], i - 1 < 0 ? COLOR_MAP.get("black")[i] : COLOR_MAP.get("brown")[i - 1]);
	}
	return newColors;
    }

    private static Color[] generateNightSeen(Color[] colors) {
	Color[] newColors = new Color[colors.length];
	for (int i = 0; i < colors.length; i++) {
	    newColors[i] = i + 1 < colors.length ? mixColors(colors[i + 1], COLOR_MAP.get("blue")[i + 1]) : COLOR_MAP.get("white")[0];
	}
	return newColors;
    }
    
    private static Color[] generateNightUnseen(Color[] colors) {
	Color[] newColors = new Color[colors.length];
	for (int i = 0; i < colors.length; i++) {
	    newColors[i] = i - 1 < 0 ? mixColors(COLOR_MAP.get("blue")[0], COLOR_MAP.get("black")[0]) : COLOR_MAP.get("blue")[i - 1];
	}
	return newColors;
    }

    public static Color getColor(String name, int brightness) {
	return COLOR_MAP.get(name)[brightness];
    }

    public static Color getDayColor(String name, int brightness, boolean known) {
	if (known)
	    return COLORS_DAY_UNSEEN.get(name)[brightness];
	else
	    return COLORS_DAY_SEEN.get(name)[brightness];
    }

    public static Color getEveningColor(String name, int brightness, boolean known) {
	if (known)
	    return COLORS_EVENING_UNSEEN.get(name)[brightness];
	else
	    return COLORS_EVENING_SEEN.get(name)[brightness];
    }

    public static Color getNightColor(String name, int brightness, boolean known) {
	if (known)
	    return COLORS_NIGHT_UNSEEN.get(name)[brightness];
	else
	    return COLORS_NIGHT_SEEN.get(name)[brightness];
    }

    public static Color getNightColor(String name, int brightness) {
	return COLORS_NIGHT_SEEN.get(name)[brightness];
    }

    private static Color mixColors(Color c1, Color c2) {
	int nRed = (c1.getRed() + c2.getRed()) / 2;
	int nGreen = (c1.getGreen() + c2.getGreen()) / 2;
	int nBlue = (c1.getBlue() + c2.getBlue()) / 2;
	return new Color(nRed, nGreen, nBlue);
    }

    public static Color moreGrey(Color color) {
	int red = color.getRed();
	int green = color.getGreen();
	int blue = color.getBlue();
	int grey = 40;
	int newRed = (2 * red + grey) / 3;
	int newGreen = (2 * green + grey) / 3;
	int newBlue = (2 * blue + grey) / 3;
	Color newColor = new Color(newRed, newGreen, newBlue);
	return newColor;
    }
}
