package util;

import java.awt.event.KeyEvent;
import java.util.ArrayList;

public class Dir {

    public static ArrayList<Dir> DIRS = new ArrayList<Dir>();
    public static final Dir LEFT = new Dir(-1, 0);
    public static final Dir RIGHT = new Dir(1, 0);
    public static final Dir UP = new Dir(0, -1);
    public static final Dir DOWN = new Dir(0, 1);
    public static final Dir UP_LEFT = new Dir(-1, -1);
    public static final Dir DOWN_LEFT = new Dir(-1, 1);
    public static final Dir UP_RIGHT = new Dir(1, -1);
    public static final Dir DOWN_RIGHT = new Dir(1, 1);
    public static final Dir NONE = new Dir(0, 0);

    private int deltaX, deltaY;

    private Dir(int deltaX, int deltaY) {
	this.deltaX = deltaX;
	this.deltaY = deltaY;
	DIRS.add(this);
    }

    public static Dir randomDir() {
	return DIRS.get(Util.randInt(0, 7));
    }
    
    public Dir getOppositeDir() {
	int dX = -deltaX;
	int dY = -deltaY;
	
	for (Dir cand : DIRS) {
	    if (cand.getDeltaX() == dX && cand.getDeltaY() == dY) {
		return cand;
	    }
	}
	return null;
    }
    
    public static Dir getDir(int dx, int dy) {
	for (Dir cand : DIRS) {
	    if (cand.getDeltaX() == dx && cand.getDeltaY() == dy) {
		return cand;
	    }
	}
	return null;
    }

    public static Dir keyToDir(KeyEvent e) {
	switch (e.getKeyCode()) {
	case KeyEvent.VK_NUMPAD1:
	case KeyEvent.VK_B:
	    return Dir.DOWN_LEFT;
	case KeyEvent.VK_NUMPAD2:
	case KeyEvent.VK_N:
	    return Dir.DOWN;
	case KeyEvent.VK_NUMPAD3:
	case KeyEvent.VK_M:
	    return Dir.DOWN_RIGHT;
	case KeyEvent.VK_NUMPAD4:
	case KeyEvent.VK_G:
	    return Dir.LEFT;
	case KeyEvent.VK_NUMPAD5:
	case KeyEvent.VK_H:
	    return Dir.NONE;
	case KeyEvent.VK_NUMPAD6:
	case KeyEvent.VK_J:
	    return Dir.RIGHT;
	case KeyEvent.VK_NUMPAD7:
	case KeyEvent.VK_T:
	    return Dir.UP_LEFT;
	case KeyEvent.VK_NUMPAD8:
	case KeyEvent.VK_Y:
	    return Dir.UP;
	case KeyEvent.VK_NUMPAD9:
	case KeyEvent.VK_U:
	    return Dir.UP_RIGHT;
	}
	return null;
    }

    public int getDeltaX() {
	return this.deltaX;
    }

    public int getDeltaY() {
	return this.deltaY;
    }
    
    public String toString() {
	return "dx: " + deltaX + " dy: " + deltaY;
    }
  

}
