package util;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Random;

public class Util {

    private static final boolean DEBUG = true;
    private static Random rand = new Random();

    public static String splitToLines(String text, int length) {
	text = text.replaceAll("\n", " ");
	if (text.length() <= length)
	    return text;
	for (int i = length; i >= 0; i--) {
	    if (text.charAt(i) == ' ') {
		return text.substring(0, i) + "\n" + splitToLines(text.substring(++i), length);
	    }
	}
	return text;
    }

    public static double getDistance(int x1, int y1, int x2, int y2) {
	return Math.sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2));
    }

    public static int randInt(int from, int to) {
	return from + rand.nextInt(to - from + 1);
    }

    public static int random(int n, double factor) {
	double variedN = n * (1 + (Math.random() - 0.5) * factor);
	return (int) Math.round(variedN);
    }

    public static String capitalizeFirstLetter(String s) {
	return s.substring(0, 1).toUpperCase() + s.substring(1);
    }

    public static void log(String s) {
	if (DEBUG)
	System.out.println(s);
    }

    public static void logError(String s) {
	if (DEBUG)
	System.err.println(s);

	try {
	    BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream("errlog.txt", true), "UTF-8"));
	    writer.append(s);
	    writer.newLine();
	    writer.close();
	} catch (Exception e) {
	    System.err.println("Couldn't write to error log");
	}
    }

    public static ArrayList<String[]> getPrototypeAttributes(String filename, String separator) {
	int lineCount = 1;
	try {
	    BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(filename), "UTF-8"));

	    ArrayList<String[]> protos = new ArrayList<String[]>();
	    String line;
	    while ((line = reader.readLine()) != null) {
		String[] attributes = line.split(separator);
		protos.add(attributes);
		lineCount++;
	    }
	    reader.close();
	    return protos;
	} catch (Exception e) {
	    System.err.println("Couldn't read prototype from " + filename + " at line " + lineCount + ".");
	}
	return null;
    }
}
