package views;

import items.Item;

import java.awt.Graphics2D;

import maps.Cell;
import maps.Map;
import models.GameModel;
import skills.Skill;
import spells.Spell;
import util.ColorData;
import chars.Char;
import chars.Player;

public class GameView extends View {

    public static final int STATS_WIDTH = 20;
    public static final int AREA_WIDTH = WIDTH - STATS_WIDTH - 1;
    public static final int AREA_HEIGHT = HEIGHT - GameModel.OUTPUTBOX_HEIGHT - 1;
    private static final int BORDER_X = STATS_WIDTH + 1;
    private static final int BORDER_Y = GameModel.OUTPUTBOX_HEIGHT + 1;
    private boolean smallMapX;
    private boolean smallMapY;
    private int xShift;
    private int yShift;
    private int xLeftBound, xRightBound;
    private int yLeftBound, yRightBound;

    public GameView(GameModel model) {
	super(model);
    }

    public void paint(Graphics2D g) {
	drawBackground(g);
	drawFrames(g);
	drawOutputBox(g);
	drawStats(g);
	drawArea(g);
    }

    private void drawFrames(Graphics2D g) {
	g.setColor(ColorData.getColor("brown", 3));
	for (int i = 0; i < STATS_WIDTH; i++) {
	    drawCharAt(g, 'X', i, GameModel.OUTPUTBOX_HEIGHT);
	}
	drawCharAt(g, 'O', STATS_WIDTH, GameModel.OUTPUTBOX_HEIGHT);
	for (int i = STATS_WIDTH + 1; i < WIDTH; i++) {
	    drawCharAt(g, 'X', i, GameModel.OUTPUTBOX_HEIGHT);
	}

	for (int i = GameModel.OUTPUTBOX_HEIGHT + 1; i < HEIGHT; i++) {
	    drawCharAt(g, 'X', STATS_WIDTH, i);
	}
    }

    private void drawOutputBox(Graphics2D g) {
	String visibleOutput = ((GameModel) model).getVisibleOutput();

	g.setColor(ColorData.getColor("grey", 3));
	drawStringAt(g, visibleOutput, 0, 0);
    }

    private void drawStats(Graphics2D g) {
	Player player = ((GameModel) model).getPlayer();

	g.setColor(ColorData.getColor("grey", 4));
	drawStringAt(g, player.getName(), 1, GameModel.OUTPUTBOX_HEIGHT + 2);
	g.setColor(ColorData.getColor("grey", 3));
	drawStringAt(g, "Level " + player.getLevel() + " " + player.getRace().getName(), 1, GameModel.OUTPUTBOX_HEIGHT + 3);
	drawStringAt(g, "HP: " + (int) player.getHp() + "/" + (int) player.getMaxHp(), 1, GameModel.OUTPUTBOX_HEIGHT + 7);
	drawStringAt(g, "MP: " + (int) player.getMp() + "/" + (int) player.getMaxMp(), 1, GameModel.OUTPUTBOX_HEIGHT + 8);
	drawStringAt(g, "Speed: " + player.getMovingSpeed(), 1, GameModel.OUTPUTBOX_HEIGHT + 9);

	drawStringAt(g, "Damage: " + player.getMinDmg() + "-" + player.getMaxDmg(), 1, GameModel.OUTPUTBOX_HEIGHT + 11);
	drawStringAt(g, "Defense: " + player.getDefense(), 1, GameModel.OUTPUTBOX_HEIGHT + 12);
	drawStringAt(g, "Sorcery: " + player.getSorcery(), 1, GameModel.OUTPUTBOX_HEIGHT + 13);
	// Omaa juttua
	drawStringAt(g, "Turn: " + ((GameModel) model).getTurn(), 1, GameModel.OUTPUTBOX_HEIGHT + 19);
	drawStringAt(g, "Time: " + ((GameModel) model).getTime(), 1, GameModel.OUTPUTBOX_HEIGHT + 20);
	drawStringAt(g, "Day: " + ((GameModel) model).getDay(), 1, GameModel.OUTPUTBOX_HEIGHT + 21);
	drawStringAt(g, ((GameModel) model).getTimeOfDayWord(), 1, GameModel.OUTPUTBOX_HEIGHT + 22);
	drawStringAt(g, ((GameModel) model).getHourOutput(), 1, GameModel.OUTPUTBOX_HEIGHT + 23);

	for (int i = 1; i < STATS_WIDTH - 1; i++) {
	    char c;
	    if ((double) player.getExp() / (double) player.getExpNeeded() >= (double) i / (double) (STATS_WIDTH - 2))
		c = '+';
	    else
		c = '-';
	    drawCharAt(g, c, i, GameModel.OUTPUTBOX_HEIGHT + 5);
	}

	int y = 0;
	for (Skill s : Skill.getSkills()) { // TODO: use getLearnedSkills()?
	    if (s.isLearned()) {
		drawStringAt(g, s.getName() + ": " + s.getLevel(), 1, GameModel.OUTPUTBOX_HEIGHT + 15 + y);
		y++;
	    }
	}

	drawStringAt(g, "F10 - Help", 1, HEIGHT - 2);
    }

    public void initMapView(Map map) {
	int mapWidth = map.getWidth();
	int mapHeight = map.getHeight();
	if (AREA_WIDTH >= mapWidth) {
	    smallMapX = true;
	    xShift = (AREA_WIDTH - mapWidth) / 2;
	    xLeftBound = 0;
	    xRightBound = mapWidth;
	} else {
	    smallMapX = false;
	}
	if (AREA_HEIGHT >= mapHeight) {
	    smallMapY = true;
	    yShift = (AREA_HEIGHT - mapHeight) / 2;
	    yLeftBound = 0;
	    yRightBound = mapHeight;
	} else {
	    smallMapY = false;
	}
    }

    private void drawArea(Graphics2D g) { // TODO: more consistent coding needed
	Player player = ((GameModel) model).getPlayer();
	Map currentMap = ((GameModel) model).getCurrentMap();
	Cell[][] cells = currentMap.getCells();

	int pX = player.getCell().getX();
	int pY = player.getCell().getY();

	int xMax = cells.length;
	int yMax = cells[0].length;
	if (!smallMapX) {
	    xShift = 0;
	    xLeftBound = pX - AREA_WIDTH / 2;
	    xRightBound = pX + AREA_WIDTH / 2;
	    if (xLeftBound < 0) {
		xShift = -xLeftBound;
		xLeftBound = 0;
		xRightBound += xShift;
	    } else if (xRightBound > xMax) {
		xShift = xMax - xRightBound;
		xLeftBound += xShift;
		xRightBound = xMax;
	    }
	}
	if (!smallMapY) {
	    yShift = 0;
	    yLeftBound = pY - AREA_HEIGHT / 2;
	    yRightBound = pY + AREA_HEIGHT / 2;
	    if (yLeftBound < 0) {
		yShift = -yLeftBound;
		yLeftBound = 0;
		yRightBound += yShift;
	    } else if (yRightBound > yMax) {
		yShift = yMax - yRightBound;
		yLeftBound += yShift;
		yRightBound = yMax;
	    }
	}

	Cell cell = null;
	for (int mapY = yLeftBound; mapY < yRightBound; mapY++) {
	    for (int mapX = xLeftBound; mapX < xRightBound; mapX++) {
		cell = cells[mapX][mapY];
		if (cell.getVisibility() == Cell.UNKNOWN)
		    ;
		else if (cell.getVisibility() == Cell.KNOWN) {
		    drawGround(cell, g);
		} else if (cell.getChar() != null) {
		    drawChar(cell.getChar(), g);
		} else if (cell.getItems().size() > 0) {
		    drawItem(cell.getItems().get(cell.getItems().size() - 1), g);
		} else
		drawGround(cell, g);
	    }
	}
	drawSpells(g);
    }

    private void drawGround(Cell c, Graphics2D g) { // TODO: move to view
	int drawX = getDrawX(c.getX());
	int drawY = getDrawY(c.getY());
	g.setColor(c.getInstantColor());
	View.drawCharAt(g, c.getGroundProto().getSymbol(), drawX, drawY);
    }

    private void drawChar(Char c, Graphics2D g) {
	int drawX = getDrawX(c.getCell().getX());
	int drawY = getDrawY(c.getCell().getY());
	g.setColor(c.getColor());		// TODO: loppuun
	View.drawCharAt(g, c.getSymbol(), drawX, drawY);
    }
    
    private void drawItem(Item i, Graphics2D g) {
	int drawX = getDrawX(i.getCell().getX());
	int drawY = getDrawY(i.getCell().getY());
	g.setColor(i.getInstantColor());
	View.drawCharAt(g, i.getSymbol(), drawX, drawY);
    }

    private void drawSpells(Graphics2D g) {
	Map currentMap = ((GameModel) model).getCurrentMap();

	for (Spell s : currentMap.getAffectingSpells()) {
	    s.draw(g);
	}
    }
    
    public void clearAt(Graphics2D g, Cell cell) {
	g.setColor(COLOR_BLACK);
	g.fillRect(getDrawX(cell.getX()) * CELL_WIDTH, getDrawY(cell.getY()) * CELL_HEIGHT, CELL_WIDTH, CELL_HEIGHT);
    }

    public void drawCharAt(Graphics2D g, char c, Cell cell) {
	g.drawString(Character.toString(c), getDrawX(cell.getX()) * CELL_WIDTH, (getDrawY(cell.getY()) + 1) * CELL_HEIGHT);
    }

    private int getDrawX(int x) {
	if (smallMapX)
	    return BORDER_X + x + xShift;
	else {
	    if (xShift == 0)
		return BORDER_X + AREA_WIDTH / 2 - ((GameModel) model).getPlayer().getCell().getX() + x;
	    else if (xShift < 0)
		return BORDER_X + AREA_WIDTH / 2 - ((GameModel) model).getPlayer().getCell().getX() + x - xShift;
	    else
		return BORDER_X + x;
	}
    }

    private int getDrawY(int y) {
	if (smallMapY)
	    return BORDER_Y + y + yShift;
	else {
	    if (yShift == 0)
		return BORDER_Y + 1 + AREA_HEIGHT / 2 - ((GameModel) model).getPlayer().getCell().getY() + y;
	    else if (yShift < 0)
		return BORDER_Y + 1 + AREA_HEIGHT / 2 - ((GameModel) model).getPlayer().getCell().getY() + y - yShift;
	    else
		return BORDER_Y + 1 + y;
	}
    }

}
