package views;

import items.Item;
import items.WieldSpot;

import java.awt.Graphics2D;
import java.util.ArrayList;

import lists.ItemList;
import lists.ListItem;
import models.InventoryModel;
import chars.Player;

public class InventoryView extends View {

    public InventoryView(InventoryModel model) {
	super(model);
    }

    public void paint(Graphics2D g) {
	drawBackground(g);
	drawTitle(g, "INVENTORY");
	drawList(g);
    }

    private void drawList(Graphics2D g) {
	ItemList itemList = ((InventoryModel) model).getItemList();
	int action = ((InventoryModel) model).getAction();
	Player player = ((InventoryModel) model).getPlayer();

	g.setColor(COLOR_GREY);

	drawStringAt(g, "A total of " + itemList.size() + " items with total weight of " + InventoryModel.decFormat.format(player.getItemsWeight()) + " kilograms.", 2, 2);

	if (action == InventoryModel.ACTION_USE)
	    g.setColor(COLOR_LIGHTER_GREY);
	else
	    g.setColor(COLOR_GREY);
	drawStringAt(g, "1 - Use", WIDTH - 24, 4);
	if (action == InventoryModel.ACTION_WIELD)
	    g.setColor(COLOR_LIGHTER_GREY);
	else
	    g.setColor(COLOR_GREY);
	drawStringAt(g, "2 - Wield", WIDTH - 24, 5);
	if (action == InventoryModel.ACTION_DROP)
	    g.setColor(COLOR_LIGHTER_GREY);
	else
	    g.setColor(COLOR_GREY);
	drawStringAt(g, "3 - Drop", WIDTH - 24, 6);

	g.setColor(COLOR_GREY);
	drawStringAt(g, "WIELDED", 2, 4);

	g.setColor(COLOR_DARKER_YELLOW);
	drawStringAt(g, "Head", 3, 6);
	drawStringAt(g, "Chest", 3, 7);
	drawStringAt(g, "Legs", 3, 8);
	drawStringAt(g, "Feet", 3, 9);
	drawStringAt(g, "Right hand", 3, 10);
	drawStringAt(g, "Left hand", 3, 11);
	drawStringAt(g, "Finger", 3, 12);

	g.setColor(COLOR_GREY);
	ArrayList<ListItem> wieldedItems = itemList.getCurrentPage().getWieldedItems();
	for (int i = 0; i < wieldedItems.size(); i++) {
	    ListItem l = wieldedItems.get(i);
	    Item item = l.getItem();

	    int y = 6;
	    if (item.getWieldSpot() == WieldSpot.WIELD_HEAD)
		y = 6;
	    if (item.getWieldSpot() == WieldSpot.WIELD_CHEST)
		y = 7;
	    if (item.getWieldSpot() == WieldSpot.WIELD_LEGS)
		y = 8;
	    if (item.getWieldSpot() == WieldSpot.WIELD_FEET)
		y = 9;
	    if (item.getWieldSpot() == WieldSpot.WIELD_RIGHTHAND)
		y = 10;
	    if (item.getWieldSpot() == WieldSpot.WIELD_LEFTHAND)
		y = 11;
	    if (item.getWieldSpot() == WieldSpot.WIELD_FINGER)
		y = 12;

	    drawStringAt(g, l.getLetter() + ") " + item.getName() + " " + item.getAttrString(), 20, y);
	}

	ArrayList<WieldSpot> wieldSpots = WieldSpot.getWieldSpots();
	for (int i = 1; i < wieldSpots.size(); i++) {
	    WieldSpot w = wieldSpots.get(i);
	    if (w.getWieldedItem() == null) {
		drawStringAt(g, "-", 20, 5 + i);
	    }

	}

	g.setColor(COLOR_GREY);
	String pageString = (itemList.getNumberOfPages() > 1 ? (" (page " + itemList.getCurrentPageNumber() + "/" + itemList.getNumberOfPages() + ")") : "");
	drawStringAt(g, "IN POSSESSION" + pageString, 2, 15);

	int y = 0;
	ArrayList<ListItem> listItems = itemList.getCurrentPage().getListItems();
	for (int i = itemList.getWieldedItems().size(); i < listItems.size(); i++) {
	    ListItem l = listItems.get(i);
	    Item item = l.getItem();

	    drawStringAt(g, l.getLetter() + ") " + item.getName() + " " + item.getAttrString(), 3, 17 + y);

	    if (l.isSeparatorHere())
		y++;
	    y++;
	}

	drawStringAt(g, "Up arrow - Previous items", WIDTH - 24, 8);
	drawStringAt(g, "Down arrow - Next items", WIDTH - 24, 9);
    }
}
