package views;

import java.awt.Color;
import java.awt.Graphics2D;

import models.Model;
import ui.Window;
import util.ColorData;

public abstract class View {

    public static final int WIDTH = 80;
    public static final int HEIGHT = 38;

    static final int CELL_WIDTH = 10;
    static final int CELL_HEIGHT = 14;

    protected static final Color COLOR_GREY = ColorData.getColor("grey", 3);
    protected static final Color COLOR_LIGHTER_GREY = ColorData.getColor("grey", 4);
    protected static final Color COLOR_DARKER_YELLOW = ColorData.getColor("yellow", 2);
    protected static final Color COLOR_DARKER_RED = ColorData.getColor("red", 2);
    protected static final Color COLOR_BLACK = ColorData.getColor("black", 0);

    protected Model model;

    public View(Model model) {
	this.model = model;
    }

    public abstract void paint(Graphics2D g);

    public static void clearAt(Graphics2D g, int x, int y) {
	g.setColor(COLOR_BLACK);
	g.fillRect(x * CELL_WIDTH, y * CELL_HEIGHT, CELL_WIDTH, CELL_HEIGHT);
    }

    public static void drawCharAt(Graphics2D g, char c, int x, int y) {
	g.drawString(Character.toString(c), x * CELL_WIDTH, (y + 1) * CELL_HEIGHT);
    }

    public static void drawStringAt(Graphics2D g, String s, int x, int y) {
	String[] lines = s.split("\n");
	for (int i = 0; i < lines.length; i++) {
	    g.drawString(lines[i], x * CELL_WIDTH, (y + 1 + i) * CELL_HEIGHT);
	}
    }

    protected void drawBackground(Graphics2D g) {
	Window window = model.getWindow();
	g.setColor(COLOR_BLACK);
	g.fillRect(0, 0, window.getWidth(), window.getHeight());
    }

    protected void drawTitle(Graphics2D g, String title) {
	int length = (WIDTH - title.length() - 1) / 2;
	g.setColor(ColorData.getColor("brown", 3));
	for (int i = 0; i < length; i++) {
	    drawCharAt(g, 'X', i, 0);
	}
	for (int i = WIDTH - 1; i >= WIDTH - 1 - length; i--) {
	    drawCharAt(g, 'X', i, 0);
	}
	g.setColor(ColorData.getColor("grey", 4));
	drawStringAt(g, title, length + 1, 0);
    }
}
